package com.qs.mp.web.controller.api.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.excel.TicketExcel;
import com.qs.mp.admin.domain.param.BathIdParam;
import com.qs.mp.admin.domain.param.TicketBoxCreateParam;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.admin.service.ITicketService;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.param.ChannelParam;
import com.qs.mp.channel.domain.vo.ChannelVO;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelUserRelService;
import com.qs.mp.common.annotation.Log;
import com.qs.mp.common.constant.UserConstants;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.BusinessType;
import com.qs.mp.common.enums.ChannelRoleEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.common.enums.TicketBoxStatusEnum;
import com.qs.mp.common.enums.TicketTypeEnum;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.utils.ExcelUtil;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import ma.glasnost.orika.MapperFacade;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

/**
 * @auther zhongcp
 * @create 2022-02-28 16:17:48
 * @describe 盲票管理前端控制器
 */
@Api("渠道管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/ticket/box/*")
@Component
public class TicketBoxMgrController extends BaseApiController {

	@Autowired
	private ITicketBoxService ticketBoxService;

	@Autowired
	private IChannelUserRelService channelUserRelService;

	@Autowired
	private ISysUserService userService;

	@Autowired
	private ITicketService ticketService;

	@Autowired
	private MapperFacade mapperFacade;


	@Value("${wx-channel.appId}")
	private String channelAppId;


	@Value("${mp.exportUrl}")
	private String exportUrl;

	/**
	 * 查询所有盲票组列表
	 *
	 * @return
	 */
	@PostMapping("/list")
	public TableDataInfo list(@RequestBody TicketBox ticketBox) {
		startPage();
		List<TicketBox> list = ticketBoxService.list(new QueryWrapper<>());
		return getDataTable(list);
	}

	/**
	 * 新增盲票
	 *
	 * @param
	 * @return
	 */
	@ApiOperation(value = "新增盲票", notes = "管理端创建盲票")
	@PostMapping("/create")
	public AjaxResult create(@Validated @RequestBody TicketBoxCreateParam param) {
		if (param.getType() == TicketTypeEnum.OFFLINE) {
			if (param.getPkgSalePrice() == null || param.getPkgSalePrice() == 0) {
				return AjaxResult.error("采购单价必须大于0");
			}
			if (param.getPkgUnit() == null || param.getPkgUnit() == 0) {
				return AjaxResult.error("每包张数必须大于0");
			}
		}
		boolean rtn = ticketBoxService.createTicketBox(param);
		return rtn ? AjaxResult.success() : AjaxResult.error("创建失败");
	}

	/**
	 * 盲票上下架
	 *
	 * @param
	 * @return
	 */
	@ApiOperation(value = "盲票上下架", notes = "管理端盲票上下架")
	@PostMapping("/put")
	public AjaxResult putOnOff(@RequestBody TicketBox param) {
		if (StringUtils.isBlank(param.getBoxId()) || param.getStatus() == null) {
				return AjaxResult.error("参数缺失");
		}
		TicketBox ticketBox = ticketBoxService.getById(param.getBoxId());
		if (param.getStatus() == TicketBoxStatusEnum.PUT_OFF) {
			if (ticketBox.getStatus() != TicketBoxStatusEnum.PUT_ON) {
				return AjaxResult.error("当前状态不是已上架，不能下架");
			}
			ticketBoxService.update(new LambdaUpdateWrapper<TicketBox>().set(TicketBox::getStatus, TicketBoxStatusEnum.PUT_OFF)
					.eq(TicketBox::getBoxId, ticketBox.getBoxId()));
		} else if (param.getStatus() == TicketBoxStatusEnum.PUT_ON) {
			if (ticketBox.getStatus() != TicketBoxStatusEnum.PUT_OFF && ticketBox.getStatus() != TicketBoxStatusEnum.DONE) {
				return AjaxResult.error("当前状态不是待上架或已下架，不能上架");
			}
			ticketBoxService.update(new LambdaUpdateWrapper<TicketBox>().set(TicketBox::getStatus, TicketBoxStatusEnum.PUT_ON)
					.eq(TicketBox::getBoxId, ticketBox.getBoxId()));
		}

		return AjaxResult.success("操作成功");
	}

	/**
	 * 盲票删除
	 *
	 * @param
	 * @return
	 */
	@ApiOperation(value = "盲票删除", notes = "管理端盲票删除")
	@PostMapping("/remove")
	public AjaxResult remove(@RequestBody TicketBox param) {
		if (StringUtils.isBlank(param.getBoxId())) {
			return AjaxResult.error("参数缺失");
		}
		TicketBox ticketBox = ticketBoxService.getById(param.getBoxId());
		if (ticketBox.getStatus() != TicketBoxStatusEnum.DONE) {
			return AjaxResult.error("当前状态不是待上架，不能删除");
		}
		ticketBoxService.removeTicketBox(param.getBoxId());
		return AjaxResult.success("删除成功");
	}

	@Log(title = "盲票导出", businessType = BusinessType.EXPORT)
	@PostMapping("/export")
	public AjaxResult exportTicket(@RequestBody BathIdParam ids) {
		if (null == ids || null == ids.getIds() || ids.getIds().size() == 0) {
			return AjaxResult.error("请选择需要导出的盲票组");
		}
		List<TicketExcel> excelList = new ArrayList<TicketExcel>();
		List<Ticket> listAll = new ArrayList<Ticket>();
		// 首先查询要导出的数据总数
		LambdaQueryWrapper<Ticket> queryWrapper = new LambdaQueryWrapper<Ticket>();
		queryWrapper.eq(Ticket::getBoxId, ids.getIds().get(0));
		int totalSize = ticketService.count(queryWrapper);
		if (totalSize == 0) {
			return AjaxResult.error("选择的盲票组中没有可导出的数据");
		}
		if(totalSize > 1000) {
			int totalPage = totalSize%1000 == 0?totalSize/1000:totalSize/1000+1;
			for (int i = 0; i < totalPage; i++) {
				LambdaQueryWrapper<Ticket> pageQueryWrapper = new LambdaQueryWrapper<Ticket>();
				pageQueryWrapper.eq(Ticket::getBoxId, ids.getIds().get(0));
				pageQueryWrapper.last("limit "+(i+1)+", 1000");
				List<Ticket> list = ticketService.list(pageQueryWrapper);
				if(null != list && list.size() > 0 ) {
					listAll.addAll(list);
				}
			}
		}else {
			listAll = ticketService.list(queryWrapper);
		}
		
		if(null != listAll && listAll.size() > 0 ) {
			for (Ticket ticket : listAll) {
				if (null != ticket && StringUtils.isNotBlank(ticket.getSerialNo())
						&& StringUtils.isNotBlank(ticket.getDrawNum())
						&& StringUtils.isNotBlank(ticket.getCipherLuckyNum())) {
					TicketExcel ticketExcel = new TicketExcel();
					ticketExcel.setSerialNo(ticket.getSerialNo());
					ticketExcel.setUrl(exportUrl + ticket.getSerialNo());
					ticketExcel.setDrawNum(ticket.getDrawNum());
					ticketExcel.setCipherLuckyNum(ticket.getCipherLuckyNum());
					excelList.add(ticketExcel);
				}
			}
		}
		
		ExcelUtil<TicketExcel> util = new ExcelUtil<TicketExcel>(TicketExcel.class);
		return util.exportExcel(excelList, "盲票导出数据", true);
	}


    @GetMapping("/importTemplate")
    public AjaxResult importTemplate()
    {
        ExcelUtil<TicketExcel> util = new ExcelUtil<TicketExcel>(TicketExcel.class);
        return util.importTemplateExcel("盲票导出数据",true);
    }

	@Log(title = "盲票导入", businessType = BusinessType.IMPORT)
	@PostMapping("/importTicket")
	public AjaxResult importTicket(MultipartFile file) {
		// 获取文件内的盲票数据列表
		try {
			ExcelUtil<TicketExcel> util = new ExcelUtil<TicketExcel>(TicketExcel.class);
			List<TicketExcel> ticketExcelList = util.importExcel(file.getInputStream());
			if(null == ticketExcelList || ticketExcelList.size() == 0) {
				return AjaxResult.error("导入失败，选择的导入文件中没有有效的数据");
			}
			int successNum = ticketService.importTicket(ticketExcelList);
			if(successNum == 0) {
				return AjaxResult.error("导入失败，请联系管理员");
			}
			return AjaxResult.success("导入成功， 导入数据条数："+successNum);
			// 数据处理 TODO
		} catch (Exception e) {
			return AjaxResult.error("导入失败，请联系管理员");
		}
	}


}
